import json
import os.path
from django import http

from .templatetags.jsform import get_errormessage_id

def d(text):
    return json.dumps(text)

class JSResponse(http.HttpResponse):
    '''
        HttpResponse subclass that:
            - sets correct content_type
            - can be added together (to concatenate the javascript statements in each)
    '''
    def __init__(self, content):
        self._string_content = content
        super().__init__(content, content_type='text/javascript')
    def __add__(self, other):
        return JSResponse(';'.join([self._string_content, other._string_content]))

def js_response(content):
    '''
        Kept for backward compatibility.
        Users might as well use JSResponse directly.
    '''
    return JSResponse(content)

def reload():
    return js_response('location.reload()')

def go_back():
    '''
        This is our recommend response for "Edit" form pages.
        Use with some means of "stale page detection" to trigger the previous page to reload.

        We officially endorse NavTricks.js for "smart back" behaviour.
        https://github.com/quadrant-newmedia/NavTricks

        TODO - add backOrUp([internal_hosts]) function to NavTricks?
    '''
    return js_response(f'''
        if (window.NavTricks) {{
            if (NavTricks.previousPageIsInternal()) {{
                NavTricks.returnToPreviousPage()
            }}
            else {{
                NavTricks.withParentPage(function(p) {{
                    location = p.path
                }})
            }}
        }}
        else {{
            history.back();
        }}
    ''')
    return js_response('history.back()')
def go_to(url):
    '''
        Useful if you have an in-page "Add" form on a list page, and want to navigate to detail page for the created object upon form completion.
    '''
    return js_response(f'location = {d(url)}')
def replace_location(url):
    '''
        NOT RECOMMENDED. 
    '''
    return js_response(f'''
        location.replace({d(url)});
    ''')

def unblock_form():
    return js_response('''form.removeAttribute('block-submissions')''')

def _optional_unblock(unblock):
    return unblock_form() if unblock else js_response('');
def _optional(possible_js_response):
    '''
        Helper function which makes it easier to (conditionally) add different JSResponse objects together
    '''
    if (possible_js_response) :
        return possible_js_response
    return JSResponse('');

def alert(message, allow_further_submissions=False):
    '''
        Useful for error messages that aren't really typical form error messages, or for actions that are triggered by a simple button push (where there is no corresponding container for error messages).

        Note - allow_further_submissions is so-called for backward compatibility. In other functions, we've renamed it to "unblock".
    '''
    return js_response(f'alert({d(message)});') + _optional_unblock(allow_further_submissions)

def _get_script_content(script_name):
    with open(os.path.join(os.path.dirname(__file__), 'js_helpers', script_name)) as f :
        return f.read()

'''
    Form manipulation helpers.

    Note that we generally recommend using jsform_elementmerge.js and simply returning html pages to be merged into the current document.

    In particular circumstances, you may find it easier to mutate the form directly, and you can use these.

    Note that all of these functions return true, to allow further form submissions.
'''
def reset_form_inputs(unblock=True):
    '''
        Resets form and unblocks forms
    '''
    return js_response(f'form.reset();')+_optional_unblock(unblock)
def clear_form_errors(unblock=True):
    '''
        Requires django
        Resets and unblocks form.
    '''
    return js_response(f'''
        {_get_script_content("clear_form_errors.js")};
        clear_form_errors(form); 
    ''')+_optional_unblock(unblock)
def set_form_errors(form, status_code=400, unblock=True, focus_errors=True):
    return set_raw_form_errors(
        form.non_field_errors(), 
        {get_errormessage_id(field): field.errors for field in form},
        status_code,
        unblock=unblock,
        focus_errors=focus_errors,
    )
def set_raw_form_errors(form_errors, error_map, status_code=400, unblock=True, focus_errors=True):
    r = js_response(f'''
        {_get_script_content("set_form_errors.js")};
        set_form_errors(form, {d(form_errors)}, {d(error_map)});
    ''')+_optional_unblock(unblock)+_optional(focus_errors and focus_form_errors());
    r.status_code = status_code
    return r
def focus_form_errors():
    return js_response(f'''
        {_get_script_content("focus_form_errors.js")};
        focus_form_errors(form);
    ''')

def reset_form(unblock=True):
    return js_response(f'''
        {_get_script_content("clear_form_errors.js")};
        clear_form_errors(form); 
        form.reset();
    ''')+_optional_unblock(unblock)


import json
import time

from django import forms, template
from django.conf import settings
from django.shortcuts import resolve_url
from django.utils.html import format_html
from django.utils.safestring import mark_safe

register = template.Library()

def get_errormessage_id(bound_field):
    return f'error-{bound_field.html_name}'
def _add_widget_attr(bound_field, name, value):
    _as_widget = bound_field.as_widget
    def as_widget(widget=None, attrs=None, only_initial=False):
        attrs = attrs or {}
        attrs[name] = value
        return _as_widget(widget=widget, attrs=attrs, only_initial=only_initial)
    bound_field.as_widget = as_widget

@register.filter
def is_single_checkbox(bound_field):
    return isinstance(bound_field.field, forms.BooleanField)

@register.filter
def with_error_message_container(bound_field):
    return format_html('''
        {field}
        {error_message_container}
    ''', field=with_error_attributes(bound_field), error_message_container=error_message_container(bound_field))

@register.filter
def with_error_attributes(bound_field):
    '''
        Can be used (in conjunction with error_message_container) when with_error_message_container is not flexible enough.

        See how form_fields.html uses this to render checkboxes.
    '''
    error_message_id = get_errormessage_id(bound_field)
    bound_field._error_message_id = error_message_id
    _add_widget_attr(bound_field, 'aria-errormessage', error_message_id)
    if bound_field.errors :
        _add_widget_attr(bound_field, 'aria-invalid', 'true')
    return bound_field
@register.filter
def error_message_container(bound_field):
    '''
        Important: we always generate the error message container, even if empty.

        This is important, because aria-live only works for elements that are initially in the DOM.

        It also makes it easier to add error messages later via js (see how js_response.set_form_errors() works).

        Lastly, jsform_elementmerge.js doesn't work as desired if you have "optional" elements with any subsequent siblings, so it's always best to put them in their own container.
    '''
    try :
        error_message_id = bound_field._error_message_id
    except AttributeError :
        raise Exception('You must call "with_error_attributes(bound_field)" on the bound field before you call this function (or the field won\'t be rendered with the correct attributes to link it to this error message container')
    return format_html('''\
        <div class="error" aria-live="polite" id="{error_message_id}">{errors}</div>
    ''', error_message_id=error_message_id, errors='. '.join(bound_field.errors))

@register.filter
def with_widget_attr(bound_field, attr):
    # Note - this MUTATES the bound_field
    # You can call this as many times as you like, with different attributes
    # Can be used sort of like django-widget-tweaks to add arbitrary widget attributes from the template
    name, value = attr.split(',', 1)
    _add_widget_attr(bound_field, name, value)
    return bound_field


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('VERSION', 'r') as fh:
    version = fh.read()

setuptools.setup(
    name="django_jsform",
    version=version,
    author="Alex Fischer",
    author_email="alex@quadrant.net",
    description="Django integration for jsform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/quadrant-newmedia/django_jsform",
    packages=['django_jsform'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=["Django>=2.2,<3.3"],
    include_package_data=True,
)